﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using System.ComponentModel.DataAnnotations;

namespace FlexGridFilterIssues
{
    public class Person : INotifyPropertyChanged
    {
        #region Fields

        int _id, _countryID;
        string _first, _last;

        #endregion

        #region ctor

        public Person(int id)
        {
            ID = id;
        }

        #endregion

        #region Object Model

        [Display(Name = "ID")]
        public int ID
        {
            get { return _id; }
            set
            {
                if (value != _id)
                {
                    _id = value;
                    RaisePropertyChanged("ID");
                }
            }
        }

        [Display(Name = "Name")]
        public string Name
        {
            get { return string.Format("{0} {1}", First, Last); }
        }

        [Display(Name = "CountryName")]
        public string CountryName
        {
            get { return Country.CountryNames[_countryID]; }
        }

        [Display(Name = "Country")]
        public Country Country
        {
            get { return new Country(_countryID); }
        }

        [Display(Name = "CountryID")]
        public int CountryID
        {
            get { return _countryID; }
            set
            {
                if (value != _countryID && value > -1 && value < Country.CountryNames.Length)
                {
                    _countryID = value;
                    RaisePropertyChanged(null);
                }
            }
        }

        [Display(Name = "First")]
        public string First
        {
            get { return _first; }
            set
            {
                if (value != _first)
                {
                    _first = value;
                    RaisePropertyChanged(null);
                }
            }
        }

        [Display(Name = "Last")]
        public string Last
        {
            get { return _last; }
            set
            {
                if (value != _last)
                {
                    _last = value;
                    RaisePropertyChanged(null);
                }
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        void RaisePropertyChanged(string propertyName)
        {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }

        #endregion
    }

    public class Country : INotifyPropertyChanged
    {
        #region Fields

        int _id;
        string _name;
        public static string[] CountryNames = "China|India|United States|Indonesia|Brazil|Pakistan|Bangladesh|Nigeria|Russia|Japan|Mexico|Philippines|Vietnam|Germany|Ethiopia|Egypt|Iran|Turkey|Congo|France|Thailand|United Kingdom|Italy|Myanmar".Split('|').OrderBy(x=>x).ToArray();

        #endregion

        #region ctor

        public Country(int id)
        {
            ID = id;
            Name = CountryNames[id];
        }

        #endregion

        #region Object Model

        [Display(Name = "ID")]
        public int ID
        {
            get { return _id; }
            set
            {
                if (value != _id)
                {
                    _id = value;
                    RaisePropertyChanged("ID");
                }
            }
        }

        [Display(Name = "Name")]
        public string Name
        {
            get { return _name; }
            set
            {
                if (value != _name)
                {
                    _name = value;
                    RaisePropertyChanged("Name");
                }
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        void RaisePropertyChanged(string propertyName)
        {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }

        #endregion
    }

    public class SampleData : ObservableCollection<Person>
    {
        #region Data Generators

        static Random _rnd = new Random();
        string[] _firstNames = "Andy|Ben|Charlie|Dan|Ed|Fred|Gil|Herb|Jack|Karl|Larry|Mark|Noah|Oprah|Paul|Quince|Rich|Steve|Ted|Ulrich|Vic|Xavier|Zeb".Split('|');
        string[] _lastNames = "Ambers|Bishop|Cole|Danson|Evers|Frommer|Griswold|Heath|Jammers|Krause|Lehman|Myers|Neiman|Orsted|Paulson|Quaid|Richards|Stevens|Trask|Ulam".Split('|');

        string GetString(string[] arr)
        {
            return arr[_rnd.Next(arr.Length)];
        }
        #endregion

        public SampleData()
        {
            for (int i = 0; i < 50; i++)
            {
                Add(new Person(i)
                {
                    First = GetString(_firstNames),
                    Last = GetString(_lastNames),
                    CountryID = _rnd.Next() % Country.CountryNames.Length,
                });
            }
        }
    }
}
